from InquirerPy import inquirer
from typing import Union, Optional


def select(_choices: Union[dict, list, str, None]) -> Optional[str]:
    if _choices is None or isinstance(_choices, str):
        return _choices

    choices = []
    if isinstance(_choices, list):
        for i, c in enumerate(_choices):
            if isinstance(c, list):
                choices.append({
                    "name": f"[{i}]: [[list]]",
                    "value": c
                })
            elif isinstance(c, dict):
                choices.append({
                    "name": f"[{i}]: [[dict]]",
                    "value": c
                })
            else:
                choices.append({
                    "name": f"[{i}]: {c}",
                    "value": c
                })
    elif isinstance(_choices, dict):
        for key, value in _choices.items():
            if isinstance(value, list):
                choices.append({
                    "name": f"{key}: [[list]]",
                    "value": value
                })
            elif isinstance(value, dict):
                choices.append({
                    "name": f"{key}: [[dict]]",
                    "value": value
                })
            else:
                choices.append({
                    "name": f"{key}: {value}",
                    "value": value
                })

    prompt = inquirer.select(
        message="Ambiguous choices, please select one:",
        choices=choices,
    )

    @prompt.register_kb("q")
    def _(event):
        event.app.exit(result=None)

    return select(prompt.execute())



import yaml
from utils.console import error
from typing import Union, Optional


def join_constructor(
    loader: yaml.Loader,
    node: yaml.nodes.SequenceNode,
    sym: str,
) -> str:
    return sym.join(loader.construct_sequence(node))


def add_constructors() -> None:
    yaml.add_constructor(
        '!and',
        lambda loader, node: join_constructor(loader, node, ' && '),
        Loader=yaml.SafeLoader
    )
    yaml.add_constructor(
        '!or',
        lambda loader, node: join_constructor(loader, node, ' || '),
        Loader=yaml.SafeLoader
    )
    yaml.add_constructor(
        '!;',
        lambda loader, node: join_constructor(loader, node, ' ; '),
        Loader=yaml.SafeLoader
    )
    yaml.add_multi_constructor(
        '!:',
        lambda loader, tag_suffix, node: join_constructor(loader, node, tag_suffix),
        Loader=yaml.SafeLoader
    )


def norm_config(config) -> Union[dict, list, str]:
    if isinstance(config, list):
        return [norm_config(item) for item in config]
    elif isinstance(config, dict):
        return {key: norm_config(value) for key, value in config.items()}
    return str(config)


def load_config(file: str) -> Optional[Union[dict, list, str]]:
    add_constructors()
    try:
        with open(file, "r") as f:
            config = yaml.safe_load(f)
        if config is None:
            config = {}
        return norm_config(config)
    except FileNotFoundError:
        error(f"`{file}` not found in the current directory.")
        config = None
    except yaml.YAMLError as e:
        error(f"Error parsing `{file}`:\n{e}")
        config = None
    return config



import argparse, subprocess

from rich.panel import Panel
from rich.text import Text
from rich.syntax import Syntax

from utils.console import console, error
from utils.loader import load_config
from utils.parser import parse_extra, parse_command, parse_filename
from utils.selector import select


def main():
    parser = argparse.ArgumentParser(
        description="Run commands from `.yaml` files using unrun.",
        epilog="Example: unrun my_command",
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    parser.add_argument("key", default=None, help="The key of the command to run from `.yaml`")
    parser.add_argument("--file", "-f", default=None, help="Path to the `.yaml`")
    parser.add_argument("extra", nargs="*", help="Extra arguments to pass to the command")
    args, unknown = parser.parse_known_args()

    key, file = args.key, args.file
    extra = parse_extra(args.extra, unknown)

    filename = parse_filename(file)

    config = load_config(filename)
    if config is None:
        return
    commands = parse_command(key, config)
    if commands is None:
        return

    command = select(commands)
    if command is None:
        console.print(
            Panel(
                Text("No command selected. Exiting.", style="bold yellow"),
                title=Text("No Command Selected", style="bold yellow")
            )
        )
        return

    if extra:
        command += f" {extra}"

    console.print(
        Panel(
            Syntax(command, "bash", word_wrap=True),
            title=Text(f"Running command for key '{args.key}'", style="bold blue"),
        )
    )

    try:
        subprocess.run(command, shell=True)
    except Exception as e:
        error(
            f"An unexpected error occurred: {e}",
            title="Unexpected Error"
        )



from rich.console import Console
from rich.panel import Panel
from rich.text import Text


console = Console()


def error(message: str, title: str = "Error") -> None:
    console.print(
        Panel(
            Text(message, style="bold red"),
            title=Text(title, style="bold red")
        )
    )



from setuptools import setup, find_packages


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="unrun",
    version="0.3.1",
    author="Casper Huang",
    author_email="casper.w.huang@qq.com",
    description="A simple CLI tool to run commands from a YAML file.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/howcasperwhat/unrun",
    packages=find_packages(where="packages"),
    package_dir={'': 'packages'},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'unrun=unrun.cli:main',
        ],
    },
    install_requires=[
        'PyYAML',
        'rich',
        'InquirerPy',
    ],
    extras_require={
        "dev": [
            "pytest-cov",
            "pytest",
            "flake8",
            "twine",
            "wheel"
        ],
    },
)



from utils.console import error
from typing import Optional, Union
import os, yaml


def parse_command(key: Optional[str], config: Union[dict, list, str]) -> Optional[Union[dict, list, str]]:
    if isinstance(config, str):
        if key is None:
            return config
        else:
            return f"{config} {key}"

    if isinstance(config, list):
        return config

    keys = key.split(".")
    results = []
    command = config
    length = len(keys)

    def fn(tar: Union[dict, list, str], i: int):
        if i == length:
            results.append(tar)
        if isinstance(tar, dict):
            for j in range(i, length):
                key = ".".join(keys[i:j + 1])
                if key in tar:
                    fn(tar[key], j + 1)
    fn(command, 0)

    if not results:
        error(f"Key '{key}' not found in the object.")
        return None

    if len(results) == 1:
        results = results[0]

    return results


def parse_filename(file: Optional[str]) -> str:
    filename = file if file else os.getenv("UNRUN_FILE")
    if not filename:
        local_f = os.path.join(os.getcwd(), "unrun.config.yaml")
        global_f = os.path.expanduser("~/unrun.config.yaml")
        if os.path.exists(local_f) and os.path.isfile(local_f):
            f = local_f
        elif os.path.exists(global_f) and os.path.isfile(global_f):
            f = global_f
        else:
            f = None
        if f is not None:
            with open(f, "r") as file:
                try:
                    config = yaml.safe_load(file)
                    filename = config.get("file", "unrun.yaml")
                except yaml.YAMLError:
                    filename = "unrun.yaml"
        else:
            filename = "unrun.yaml"
    if not filename.endswith(".yaml"):
        filename += ".yaml"
    return filename


def parse_extra(extra: list, unknown: list) -> Optional[str]:
    result = []
    if extra and unknown:
        result = extra + unknown
    elif extra:
        result = extra
    elif unknown:
        result = unknown
    if not result:
        return None
    return " ".join(result)
